#version 150

/**
 * Source:
 * https://github.com/ShockMicro/VanillaDynamicEmissives
 *
 * NOTE (8Blits): Modified IMPL for 1.21.3+ support. 
 */

// Macro for transparent emissives config
#define DYNAMIC_EMISSIVE(a) return true; case a: color.rgb = emissiveColor; 

float remap_alpha(int inputAlpha) {
    if (inputAlpha == 252) return 255.0; // Checks for alpha 252 and converts all pixels of that to alpha 255.
    
    return float(inputAlpha);
}

#ifdef FSH
bool make_emissive(inout vec4 color, vec4 pureColor, vec4 tintColor) {
    vec3 emissiveColor = pureColor.rgb * tintColor.rgb;
    int alpha = int(round(pureColor.a * 255.0));
    color.a = remap_alpha(alpha) / 255.0;

    switch (alpha) {
        case 252:
            color.rgb = emissiveColor;
    #moj_import<config/dynamic_emissives_config.glsl>
        return true;
    }

    #ifndef TERRAIN
    // 2D GUI with 3D Handheld
    bool guiState = bool(FogShape == 0.0 && ProjMat[3][3] != 0.0);
    if (alpha == 250) { // 3D
        // NOTE: Need to make visible for GUI Player

        color.a = float(!guiState); // Visible for First/3rd Person
    }
    if (alpha == 248) { // 3D & Emissive
        color.rgb = emissiveColor;
        color.a = float(!guiState); // Visible for First/3rd Person
    }
    if (alpha == 249) { // 2D
        color.a = float(guiState); // Visible for HUD/Inventory; excluding GUI Viewport
        return true;
    }
    #endif

    return false;
}
#endif